@@ -304,7 +304,7 @@ static EntityEntry entity_table[] = {
 static gint
 html_g_str_case_equal (gconstpointer v, gconstpointer v2)
 {
-	return strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
+	return g_ascii_strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
 }
 
 gulong
